SystemOrganization addCategory: #'EjerciciosPoo-Measure4'!

SequenceableCollection subclass: #MeasureInterval
	instanceVariableNames: 'start stop step'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-Measure4'!

!MeasureInterval class methodsFor: 'instance creation' stamp: 'Maffongelli 6/6/2012 00:20'!
from: startInteger to: stopInteger 
	"Answer an instance of me, starting at startNumber, ending at 
	stopNumber, and with an interval increment of 1."

	^self new
		setFrom: startInteger
		to: stopInteger
		by: (Measure unit: (startInteger unit) amount: 1) ! !

!MeasureInterval class methodsFor: 'instance creation' stamp: 'Maffongelli 6/5/2012 23:29'!
from: startInteger to: stopInteger by: stepInteger 
	"Answer an instance of me, starting at startNumber, ending at 
	stopNumber, and with an interval increment of stepNumber."

	^self new
		setFrom: startInteger
		to: stopInteger
		by: stepInteger! !

!MeasureInterval class methodsFor: 'instance creation' stamp: 'Maffongelli 6/5/2012 23:29'!
new
	"Primitive. Create and answer with a new instance of the receiver
	(a class) with no indexable fields. Fail if the class is indexable. Override
	SequenceableCollection new. Essential. See Object documentation
	whatIsAPrimitive."

	<primitive: 70>
	self isVariable ifTrue: [ ^ self new: 0 ].
	"space must be low"
	OutOfMemory signal.
	^ self new  "retry if user proceeds"
! !

!MeasureInterval class methodsFor: 'instance creation' stamp: 'Maffongelli 6/5/2012 23:29'!
newFrom: aCollection 
	"Answer an instance of me containing the same elements as aCollection."

    | newInterval n |

    (n := aCollection size) <= 1 ifTrue: [
		n = 0 ifTrue: [^self from: 1 to: 0].
		^self from: aCollection first to: aCollection last].
    	newInterval := self from: aCollection first to: aCollection last
	by: (aCollection last - aCollection first) // (n - 1).
	aCollection ~= newInterval
		ifTrue: [
			"Give a second chance, because progression might be arithmetic, but = answer false"
			(newInterval hasEqualElements: aCollection) ifFalse: [
				self error: 'The argument is not an arithmetic progression']].
	^newInterval

"	Interval newFrom: {1. 2. 3}
	{33. 5. -23} as: Interval
	{33. 5. -22} as: Interval    (an error)
	(-4 to: -12 by: -1) as: Interval
	#(2 4 6) asByteArray as: Interval.
"! !

!MeasureInterval methodsFor: 'arithmetic' stamp: 'Maffongelli 6/5/2012 23:29'!
+ number

	^ start + number to: stop + number by: step! !

!MeasureInterval methodsFor: 'arithmetic' stamp: 'Maffongelli 6/5/2012 23:29'!
- number

	^ start - number to: stop - number by: step! !

!MeasureInterval methodsFor: 'comparing' stamp: 'Maffongelli 6/5/2012 23:29'!
= anObject

	^ self == anObject
		ifTrue: [true]
		ifFalse: [anObject isInterval
			ifTrue: [start = anObject first
				and: [step = anObject increment
					and: [self last = anObject last]]]
			ifFalse: [super = anObject]]! !

!MeasureInterval methodsFor: 'adding' stamp: 'Maffongelli 6/5/2012 23:29'!
add: newObject 
	"Adding to an Interval is not allowed."

	self shouldNotImplement! !

!MeasureInterval methodsFor: 'accessing' stamp: 'Maffongelli 6/5/2012 23:29'!
anyOne
	"This message will fail for an empty Interval, super would not.
	(2 to: 1) anyOne should fail because empty."

	^self at: 1! !

!MeasureInterval methodsFor: 'accessing' stamp: 'Maffongelli 6/5/2012 23:29'!
at: anInteger 
	"Answer the anInteger'th element."

	(anInteger >= 1 and: [anInteger <= self size])
		ifTrue: [^start + (step * (anInteger - 1))]
		ifFalse: [self errorSubscriptBounds: anInteger]! !

!MeasureInterval methodsFor: 'accessing' stamp: 'Maffongelli 6/5/2012 23:29'!
at: anInteger put: anObject 
	"Storing into an Interval is not allowed."

	self error: 'you can not store into an interval'! !

!MeasureInterval methodsFor: 'enumerating' stamp: 'Maffongelli 6/5/2012 23:29'!
collect: aBlock
	| nextValue result |
	result := self species new: self size.
	nextValue := start.
	1 to: result size do:
		[:i |
		result at: i put: (aBlock value: nextValue).
		nextValue := nextValue + step].
	^ result! !

!MeasureInterval methodsFor: 'enumerating' stamp: 'Maffongelli 6/5/2012 23:29'!
do: aBlock
	"Evaluate aBlock for each value of the interval.
	Implementation note: instead of repeatedly incrementing the value
		aValue := aValue + step.
	until stop is reached,
	We prefer to recompute value from start
		aValue := start + (index * step).
	This is better for floating points accuracy, while not degrading Integer and Fraction speed too much.
	Moreover, this is consistent with methods #at: and #size"

	| aValue index size |
	index := 0.
	size := self size.
	[index < size]
		whileTrue: [aValue := start + (index * step).
			index := index + 1.
			aBlock value: aValue]! !

!MeasureInterval methodsFor: 'accessing' stamp: 'Maffongelli 6/5/2012 23:29'!
extent 
	"Answer the max - min of the receiver interval."
	"(10 to: 50) extent"

	^stop - start! !

!MeasureInterval methodsFor: 'accessing' stamp: 'Maffongelli 6/5/2012 23:29'!
first 
	"Refer to the comment in SequenceableCollection|first."

	^start! !

!MeasureInterval methodsFor: 'comparing' stamp: 'Maffongelli 6/5/2012 23:29'!
hash
	"Hash is reimplemented because = is implemented."

	^(((start hash bitShift: 2)
		bitOr: stop hash)
		bitShift: 1)
		bitOr: self size! !

!MeasureInterval methodsFor: 'accessing' stamp: 'Maffongelli 6/5/2012 23:29'!
increment
	"Answer the receiver's interval increment."

	^step! !

!MeasureInterval methodsFor: 'accessing' stamp: 'Maffongelli 6/5/2012 23:29'!
indexOf: anElement startingAt: startIndex ifAbsent: exceptionBlock
	"startIndex is an positive integer, the collection index where the search is started."
	"during the computation of val , floats are only used when the receiver contains floats"

	| index val |
	(self rangeIncludes: anElement)
		ifFalse: [^ exceptionBlock value].
	val := anElement - self first / self increment.
	val isFloat
		ifTrue: [(val - val rounded) abs * 100000000 < 1
					ifTrue: [index := val rounded + 1]
					ifFalse: [^ exceptionBlock value]]
		ifFalse: [val isInteger
					ifTrue: [index := val + 1]
					ifFalse: [^ exceptionBlock value]].
	"finally, the value of startIndex comes into play:"
	^ (index between: startIndex and: self size)
		ifTrue: [index]
		ifFalse: [exceptionBlock value]! !

!MeasureInterval methodsFor: 'testing' stamp: 'Maffongelli 6/5/2012 23:29'!
isInterval

	^ true! !

!MeasureInterval methodsFor: 'self evaluating' stamp: 'Maffongelli 6/5/2012 23:29'!
isSelfEvaluating
	^ self class == Interval! !

!MeasureInterval methodsFor: 'accessing' stamp: 'Maffongelli 6/5/2012 23:45'!
last 
	"Refer to the comment in SequenceableCollection|last."

	"^stop - (stop - start \\ step)"
	^stop! !

!MeasureInterval methodsFor: 'enumerating' stamp: 'Maffongelli 6/5/2012 23:29'!
permutationsDo: aBlock
	"Repeatly value aBlock with a single copy of the receiver. Reorder the copy
	so that aBlock is presented all (self size factorial) possible permutations."
	"(1 to: 4) permutationsDo: [:each | Transcript cr; show: each printString]"

	self asArray permutationsDo: aBlock
! !

!MeasureInterval methodsFor: 'printing' stamp: 'Maffongelli 6/5/2012 23:29'!
printOn: aStream
	aStream nextPut: $(;
	 print: start;
	 nextPutAll: ' to: ';
	 print: stop.
	step ~= 1 ifTrue: [aStream nextPutAll: ' by: '; print: step].
	aStream nextPut: $)! !

!MeasureInterval methodsFor: 'accessing' stamp: 'Maffongelli 6/5/2012 23:29'!
rangeIncludes: aNumber
	"Return true if the number lies in the interval between start and stop."

	step >= 0
		ifTrue: [^ aNumber between: start and: stop]
		ifFalse: [^ aNumber between: stop and: start]
! !

!MeasureInterval methodsFor: 'removing' stamp: 'Maffongelli 6/5/2012 23:29'!
remove: newObject 
	"Removing from an Interval is not allowed."

	self error: 'elements cannot be removed from an Interval'! !

!MeasureInterval methodsFor: 'enumerating' stamp: 'Maffongelli 6/5/2012 23:29'!
reverseDo: aBlock 
	"Evaluate aBlock for each element of my interval, in reverse order.
	Implementation notes: see do: for an explanation on loop detail"

	| aValue index |
	index := self size.
	[index > 0]
		whileTrue: [
			index := index - 1.
			aValue := start + (index * step).
			aBlock value: aValue]! !

!MeasureInterval methodsFor: 'private' stamp: 'Maffongelli 6/5/2012 23:29'!
setFrom: startInteger to: stopInteger by: stepInteger

	start := startInteger.
	stop := stopInteger.
	step := stepInteger! !

!MeasureInterval methodsFor: 'accessing' stamp: 'Maffongelli 6/6/2012 12:07'!
size
	"Answer how many elements the receiver contains."

	(step unit convertionFromSelfToNormalBlock value: (step amount))  < 0
		ifTrue: [start < stop
				ifTrue: [^ 0]
				ifFalse: [^( (stop unit convertionFromSelfToNormalBlock value: (stop amount)) - (start  unit convertionFromSelfToNormalBlock value: (start amount)) //  (step unit convertionFromSelfToNormalBlock value: (step amount)) )+ 1 ]]
		ifFalse: [stop < start
				ifTrue: [^ 0]
				ifFalse: [^( (stop unit convertionFromSelfToNormalBlock value: (stop amount)) - (start  unit convertionFromSelfToNormalBlock value: (start amount)) //  (step unit convertionFromSelfToNormalBlock value: (step amount)) )+ 1 ]]! !

!MeasureInterval methodsFor: 'private' stamp: 'Maffongelli 6/5/2012 23:29'!
species

	^Array! !

!MeasureInterval methodsFor: 'printing' stamp: 'Maffongelli 6/5/2012 23:29'!
storeOn: aStream 
	aStream nextPut: $(;
	 	store: start;
	 	nextPutAll: ' to: ';
	 	store: stop.
	step ~= 1 ifTrue: [aStream nextPutAll: ' by: '; store: step].
	aStream nextPut: $)! !

!Number methodsFor: '*ejerciciospoo-measure4' stamp: 'FromerMaffongelli 6/6/2012 16:20'!
divide: aMeasure

^Measure unit: (aMeasure unit) amount: ((1/self) * ( aMeasure amount)  ) ! !

!Number methodsFor: '*ejerciciospoo-measure4' stamp: 'FromerMaffongelli 6/6/2012 16:20'!
multiply: aMeasure

^Measure unit: (aMeasure unit) amount: (self *( aMeasure amount) ) ! !

Magnitude subclass: #Measure
	instanceVariableNames: 'unit amount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-Measure4'!

!Measure class methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/6/2012 03:28'!
unit: aUnit amount: anAmount
	
	
	
	^(self new)	unit: aUnit amount: anAmount.! !

!Measure methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/6/2012 00:35'!
* anotherMeasure 
	^anotherMeasure multiply: self.! !

!Measure methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/6/2012 00:37'!
+ aMeasure 
	|selfUnit aMeasureConverted|
	
	(self unit domain = aMeasure unit domain) ifTrue:  [
	
 		^Measure unit: self unit amount: (self amount + ((aMeasure convertTo: self unit) amount) ) 
	
	] 

	ifFalse: [^'error'].! !

!Measure methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/6/2012 00:37'!
- aMeasure 
	|selfUnit aMeasureConverted|
	
	(self unit domain = aMeasure unit domain) ifTrue:  [
	
 ^Measure unit: self unit amount: (self amount - ((aMeasure convertTo: self unit) amount) ) 
	
	] 

	ifFalse: [^'error'].! !

!Measure methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/6/2012 02:43'!
/ anotherMeasure 

	^anotherMeasure divide: self.
	
"	^Measure unit: self unit amount: ( (self amount) / aSmallInteger)"
	
"/ anotherMeasure
	^ self amount / anotherMeasure convertTo: self unit amount.

"! !

!Measure methodsFor: 'as yet unclassified' stamp: 'FromerMaffongelli 6/4/2012 17:45'!
< anotherMeasure

	^ (self unit convertionFromSelfToNormalBlock value: self amount) < (anotherMeasure unit convertionFromSelfToNormalBlock value: anotherMeasure amount)! !

!Measure methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/6/2012 03:31'!
= anotherMeasure

|domainsAreTheSame convertionsAreOk|

(anotherMeasure isKindOf: Measure) ifFalse: [^false].

domainsAreTheSame := (self unit domain = anotherMeasure unit domain).

((domainsAreTheSame = false) & (self amount = 0) & (anotherMeasure amount = 0) ) ifTrue:[^true].  

convertionsAreOk := (self unit convertionFromSelfToNormalBlock value: self amount) = (anotherMeasure unit convertionFromSelfToNormalBlock value: anotherMeasure amount).

^domainsAreTheSame & convertionsAreOk.! !

!Measure methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/3/2012 20:45'!
amount

^amount.! !

!Measure methodsFor: 'as yet unclassified' stamp: 'FromerMaffongelli 6/6/2012 20:03'!
convertTo: aUnit

 

|amountConverted|

amountConverted := aUnit convertionFromNormalToSelfBlock value:  (self unit convertionFromSelfToNormalBlock value: self amount).

(aUnit domain = self unit domain) ifTrue: [^Measure unit: aUnit amount: amountConverted.] ifFalse: [ ^ 'error'   ]

! !

!Measure methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/6/2012 03:00'!
divide: anotherMeasure

|myUnitsUp myUnitsDown|

myUnitsUp := OrderedCollection new.

myUnitsDown := OrderedCollection new.

myUnitsUp add: anotherMeasure unit.

myUnitsDown add: self unit.

(myUnitsUp = myUnitsDown)ifTrue:  [^anotherMeasure amount / self amount ]
 
ifFalse:[^CompundMeasure unitsUp: myUnitsUp unitsDown: myUnitsDown  amount: (anotherMeasure amount / self amount ) ] ! !

!Measure methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/6/2012 01:35'!
hash

	|meter|
	
	meter := MeasureUnit convertionFromSelfToNormalBlock: [:x | x * 1] convertionFromNormalToSelfBlock: [:x | x * 1]  name: 'meter' domain: 'distance'. 
	
	^ self convertTo: meter. ! !

!Measure methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/6/2012 02:20'!
multiply: anotherMeasure

|unitsUp unitsDown|

unitsUp:= OrderedCollection new.

unitsUp add: self unit.
unitsUp add: anotherMeasure unit.


^CompundMeasure unitsUp: unitsUp   unitsDown: (OrderedCollection new) amount: (self amount * anotherMeasure amount) ! !

!Measure methodsFor: 'as yet unclassified' stamp: 'Maffongelli 6/4/2012 11:13'!
negated
	^Measure unit: self unit amount: (self amount * (-1) ). ! !

!Measure methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/3/2012 22:15'!
printString

	(self amount = 1 | (self unit name last = $s)) ifTrue: 
		[^self amount printString, ' ', (unit printString)]
	 ifFalse:  
		[^self amount printString, ' ', (unit printString), 's']! !

!Measure methodsFor: 'as yet unclassified' stamp: 'Maffongelli 6/5/2012 23:40'!
to: anotherMeasure

	^ MeasureInterval from: self to: anotherMeasure ! !

!Measure methodsFor: 'as yet unclassified' stamp: 'Maffongelli 6/5/2012 23:41'!
to: anotherMeasure by: stepMeasure

	^ MeasureInterval from: self to: anotherMeasure by: stepMeasure! !

!Measure methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/3/2012 20:45'!
unit

^unit.! !

!Measure methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/3/2012 19:35'!
unit: aUnit amount: anAmount

	unit:= aUnit.
	amount:= anAmount.! !

TestCase subclass: #CompundMeasure
	instanceVariableNames: 'unitsUp unitsDown amount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-Measure4'!

!CompundMeasure class methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/6/2012 02:04'!
unitsUp: aSetOfUnits unitsDown: anotherSetOfUnits amount: anAmount

^(self new)unitsUp: aSetOfUnits unitsDown: anotherSetOfUnits amount: anAmount.! !

!CompundMeasure methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/6/2012 02:34'!
* aMeasure

|myunitsUp|

myunitsUp := self unitsUp.

myunitsUp add: aMeasure unit.


^CompundMeasure unitsUp: myunitsUp unitsDown: self unitsDown amount: (self amount * aMeasure amount).! !

!CompundMeasure methodsFor: 'as yet unclassified' stamp: 'Maffongelli 6/6/2012 13:00'!
+ anotherCompoundMeasure

	^ CompundMeasure unitsUp: (self unitsUp) unitsDown:(self unitsDown)  amount: (self amount + anotherCompoundMeasure amount).! !

!CompundMeasure methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/6/2012 02:22'!
= anotherCompundMeasure

^self amount = anotherCompundMeasure amount & self unitsUp = anotherCompundMeasure unitsUp & self unitsDown = anotherCompundMeasure unitsDown! !

!CompundMeasure methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/6/2012 01:44'!
amount

^amount.
! !

!CompundMeasure methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/6/2012 02:22'!
unitsDown

^unitsDown.! !

!CompundMeasure methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/6/2012 02:22'!
unitsUp 

^unitsUp ! !

!CompundMeasure methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/6/2012 02:03'!
unitsUp: aSetOfUnits unitsDown: anotherSetOfUnits amount: anAmount

unitsUp := aSetOfUnits.
unitsDown := anotherSetOfUnits.
amount:= anAmount.! !

TestCase subclass: #MeasureTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-Measure4'!

!MeasureTest methodsFor: 'errors' stamp: 'FromerMaffongelli 6/6/2012 20:03'!
canNotConvertMeasurementException
	
	^ 'error'! !

!MeasureTest methodsFor: 'temperature units' stamp: 'PabloFromer 6/6/2012 01:30'!
celsius
	
		^MeasureUnit convertionFromSelfToNormalBlock: [:x | x * 1]  convertionFromNormalToSelfBlock: [:x | x * 1]  name: 'celsius' domain: 'temperature' . ! !

!MeasureTest methodsFor: 'distance units' stamp: 'PabloFromer 6/6/2012 01:33'!
centimeter
	
		^MeasureUnit convertionFromSelfToNormalBlock:  [:x | x * 1/100] convertionFromNormalToSelfBlock: [:x | x * 100] name: 'centimeter' domain: 'distance' . 
		
		! !

!MeasureTest methodsFor: 'time units' stamp: 'PabloFromer 6/6/2012 01:30'!
day

		^MeasureUnit convertionFromSelfToNormalBlock: [:x | x * 60 * 60 * 24] convertionFromNormalToSelfBlock: [:x | x * 1 / (60 * 60 * 24)]  name: 'day' domain: 'time' . ! !

!MeasureTest methodsFor: 'distance units' stamp: 'PabloFromer 6/6/2012 01:30'!
decameter
	
		^MeasureUnit convertionFromSelfToNormalBlock: [:x | x * 10]  convertionFromNormalToSelfBlock:  [:x | x * 1/10]  name: 'decameter' domain: 'distance'  . ! !

!MeasureTest methodsFor: 'distance units' stamp: 'PabloFromer 6/6/2012 01:30'!
decimeter
	
		^MeasureUnit convertionFromSelfToNormalBlock: [:x | x * 1/10] convertionFromNormalToSelfBlock: [:x | x * 10]  name: 'decimeter' domain: 'distance'  . ! !

!MeasureTest methodsFor: 'distance measures' stamp: 'FromerMaffongelli 5/30/2012 19:51'!
distanceExamples 

	"Quin se anima a explicar cmo funciona este mtodo? 
	Por qu est hecho as? 
	Ver distanceUnits... se puede mejorar algo entre estos dos mtodos?"
	
	| distancesCategory distancesMessageNames |
	
	distancesCategory := self class organization categoryOfElement: thisContext method selector.
	distancesMessageNames := (self class organization listAtCategoryNamed:  distancesCategory) reject: [ :aMessageName | aMessageName = thisContext method selector ].
	
	 ^ distancesMessageNames collect: [ :aMessageName | self perform: aMessageName ]
	
! !

!MeasureTest methodsFor: 'distance units' stamp: 'FromerMaffongelli 5/30/2012 19:52'!
distanceUnits 

	"Quin se anima a explicar cmo funciona este mtodo? 
	Por qu est hecho as? 
	Ver distanceUnits... se puede mejorar algo entre estos dos mtodos?"

	| unitsCategory unitsMessageNames |
	
	unitsCategory := self class organization categoryOfElement: thisContext method selector.
	unitsMessageNames := (self class organization listAtCategoryNamed:  unitsCategory) reject: [ :aMessageName | aMessageName = thisContext method selector ].
	
	 ^ unitsMessageNames collect: [ :aMessageName | self perform: aMessageName ]
	
! !

!MeasureTest methodsFor: 'compound measures' stamp: 'PabloFromer 6/6/2012 02:33'!
eightCubicMeters

 	|unitsUp unitsDown|
	
	unitsUp:=(OrderedCollection new).
	
	unitsUp add: self meter.
	
	unitsUp add: self meter.
	
	unitsUp add: self meter.
	 

	^CompundMeasure unitsUp: unitsUp unitsDown: (OrderedCollection new) amount: 8. ! !

!MeasureTest methodsFor: 'temperature units' stamp: 'PabloFromer 6/6/2012 01:31'!
fahrenheit
	
	
		^MeasureUnit convertionFromSelfToNormalBlock: [:x | (x - 32) * 5/9] convertionFromNormalToSelfBlock: [:x | (x * 9/5 ) + 32]   name: 'fahrenheit' domain: 'temperature'  . ! !

!MeasureTest methodsFor: 'compound measures' stamp: 'Maffongelli 6/6/2012 12:48'!
finalSpeedStartingAt: initialSpeed accelerating: anAcceleration during: aTimePeriod


	^ initialSpeed + (anAcceleration * aTimePeriod).
	
	"^self error: 'Implementar la formula de velocidad final: vf = vi + a * t'"! !

!MeasureTest methodsFor: 'compound measures' stamp: 'PabloFromer 6/6/2012 02:27'!
fourSquaredMeters

 	|unitsUp unitsDown|
	
	unitsUp:=(OrderedCollection new). 
	
	unitsUp add: self meter.
	
	unitsUp add: self meter.
	 

	^CompundMeasure unitsUp: unitsUp unitsDown: (OrderedCollection new) amount: 4. ! !

!MeasureTest methodsFor: 'distance units' stamp: 'PabloFromer 6/6/2012 01:31'!
hectometer
	
		^MeasureUnit convertionFromSelfToNormalBlock: [:x | x * 100] convertionFromNormalToSelfBlock: [:x | x * 1/100]  name: 'hectometer' domain: 'distance'  . ! !

!MeasureTest methodsFor: 'time units' stamp: 'PabloFromer 6/6/2012 01:31'!
hour

		^MeasureUnit  convertionFromSelfToNormalBlock: [:x | x * 60 * 60] convertionFromNormalToSelfBlock: [:x | x * 1 / (60 * 60)]   name: 'hour' domain: 'time'  . ! !

!MeasureTest methodsFor: 'temperature units' stamp: 'PabloFromer 6/6/2012 01:31'!
kelvin
	
		^MeasureUnit convertionFromSelfToNormalBlock: [:x | x - 273]  convertionFromNormalToSelfBlock: [:x | x +  273]  name: 'fahrenheit' domain: 'temperature'  . ! !

!MeasureTest methodsFor: 'distance units' stamp: 'PabloFromer 6/6/2012 01:31'!
kilometer
	
		^MeasureUnit convertionFromSelfToNormalBlock: [:x | x * 1000]  convertionFromNormalToSelfBlock: [:x | x * 1/1000]  name: 'kilometer' domain: 'distance'  . ! !

!MeasureTest methodsFor: 'objects in category' stamp: 'HernanWilkinson 4/25/2011 17:30'!
measureExamples 

	^ OrderedCollection new
		addAll: self distanceExamples;
		addAll: self temperatureExamples;
		addAll: self timeExamples;
		yourself ! !

!MeasureTest methodsFor: 'distance units' stamp: 'PabloFromer 6/6/2012 01:31'!
meter

		^MeasureUnit convertionFromSelfToNormalBlock: [:x | x * 1] convertionFromNormalToSelfBlock: [:x | x * 1]  name: 'meter' domain: 'distance'  . ! !

!MeasureTest methodsFor: 'distance units' stamp: 'PabloFromer 6/6/2012 01:31'!
millimeter
	
		^MeasureUnit convertionFromSelfToNormalBlock: [:x | x * 1/1000] convertionFromNormalToSelfBlock: [:x | x * 1000]  name: 'millimeter' domain: 'distance'  . ! !

!MeasureTest methodsFor: 'distance measures' stamp: 'PabloFromer 6/3/2012 20:34'!
minusOneMeter 
	
		
	|unit|
	
	unit:= self meter.
	
	^(Measure unit: unit amount: (-1)). ! !

!MeasureTest methodsFor: 'time units' stamp: 'PabloFromer 6/6/2012 01:31'!
minute

		^MeasureUnit  convertionFromSelfToNormalBlock: [:x | x * 60] convertionFromNormalToSelfBlock: [:x | x * 1 / (60)]   name: 'minute' domain: 'time'  . ! !

!MeasureTest methodsFor: 'objects in category' stamp: 'HernanWilkinson 4/25/2011 17:27'!
objectsInCategoryOf: aSelector

	| category messageNames |
	
	category := self class organization categoryOfElement: aSelector.
	messageNames := (self class organization listAtCategoryNamed:  category) reject: [ :aMessageName | aMessageName = aSelector ].
	
	 ^ messageNames collect: [ :aMessageName | self perform: aMessageName ]
	

	
! !

!MeasureTest methodsFor: 'temperature measures' stamp: 'PabloFromer 6/3/2012 20:35'!
oneCelsius
	
	|unit|
	
	unit:= self celsius .
	
	^(Measure unit: unit amount: 1). ! !

!MeasureTest methodsFor: 'distance measures' stamp: 'PabloFromer 6/3/2012 20:34'!
oneCentimeter
	
		
	|unit|
	
	unit:= self centimeter .
	
	^(Measure unit: unit amount: 1). ! !

!MeasureTest methodsFor: 'time measures' stamp: 'PabloFromer 6/3/2012 20:41'!
oneDay
	
	|unit|
	
	unit:= self day.
	
	^(Measure unit: unit amount: 1). ! !

!MeasureTest methodsFor: 'time measures' stamp: 'PabloFromer 6/3/2012 20:42'!
oneHour
	
	
	|unit|
	
	unit:= self hour.
	
	^(Measure unit: unit amount: 1). ! !

!MeasureTest methodsFor: 'distance measures' stamp: 'PabloFromer 6/3/2012 20:34'!
oneHundredCentimeters 
	
	|unit|
	
	unit:= self centimeter.
	
	
	^(Measure unit: unit amount: 100). ! !

!MeasureTest methodsFor: 'distance measures' stamp: 'Maffongelli 6/6/2012 12:45'!
oneHundredTwentyOneMeters
	
	|unit|
	
	unit:= self meter.
	
	^(Measure unit: unit amount: 121). ! !

!MeasureTest methodsFor: 'compound measures' stamp: 'Maffongelli 6/6/2012 12:46'!
oneHundredTwentyOneMetersPerSecond

	|unitsUp unitsDown|
	
	unitsUp:= (OrderedCollection new).
	unitsDown:= (OrderedCollection new).
		
	unitsUp add: self meter. 
	unitsDown add: self second.


	^CompundMeasure unitsUp: unitsUp unitsDown: unitsDown  amount:  121.
	"^self error: 'este metodo debe devolver la medida 121 metro/segundo'"
	
! !

!MeasureTest methodsFor: 'distance measures' stamp: 'PabloFromer 6/3/2012 20:34'!
oneMeter
	
	|unit|
	
	unit:= self meter.
	
	^(Measure unit: unit amount: 1). ! !

!MeasureTest methodsFor: 'compound measures' stamp: 'Maffongelli 6/6/2012 12:48'!
oneMeterPerSecond

	|unitsUp unitsDown|
	
	unitsUp:= (OrderedCollection new).
	unitsDown:= (OrderedCollection new).
		
	unitsUp add: self meter. 
	unitsDown add: self second.


	^CompundMeasure unitsUp: unitsUp unitsDown: unitsDown  amount:  1.

	"^self error: 'este metodo debe devolver la medida 1 metro/segundo'"! !

!MeasureTest methodsFor: 'time measures' stamp: 'PabloFromer 6/3/2012 20:42'!
oneMinute
	
	
	|unit|
	
	unit:= self minute.
	
	^(Measure unit: unit amount: 1). ! !

!MeasureTest methodsFor: 'time measures' stamp: 'PabloFromer 6/3/2012 20:42'!
oneSecond

	|unit|
	
	unit:= self second.
	
	^(Measure unit: unit amount: 1). ! !

!MeasureTest methodsFor: 'compound measures' stamp: 'PabloFromer 6/6/2012 02:26'!
oneSquaredMeter


 	|unitsUp unitsDown|
	
	unitsUp:=(OrderedCollection new).
	
	unitsUp add: self meter.
	
	unitsUp add: self meter.
	 

	^CompundMeasure unitsUp: unitsUp unitsDown: (OrderedCollection new) amount: 1. ! !

!MeasureTest methodsFor: 'compound measures' stamp: 'PabloFromer 6/6/2012 02:28'!
oneSquaredSecond


 	|unitsUp unitsDown|
	
	unitsUp:=(OrderedCollection new).
	
	unitsUp add: self second.
	 
	unitsUp add: self second.
	
	^CompundMeasure unitsUp: unitsUp unitsDown: (OrderedCollection new) amount: 1. ! !

!MeasureTest methodsFor: 'time units' stamp: 'PabloFromer 6/6/2012 01:31'!
second

		^MeasureUnit  convertionFromSelfToNormalBlock: [:x | x * 1] convertionFromNormalToSelfBlock: [:x | x * 1]   name: 'second' domain: 'time'  . ! !

!MeasureTest methodsFor: 'time measures' stamp: 'PabloFromer 6/3/2012 20:42'!
sixtyMinutes
	
	
	|unit|
	
	unit:= self minute.
	
	^(Measure unit: unit amount: 60). ! !

!MeasureTest methodsFor: 'time measures' stamp: 'PabloFromer 6/3/2012 20:43'!
sixtyOneSeconds
	
	
	|unit|
	
	unit:= self second.
	
	^(Measure unit: unit amount: 61). ! !

!MeasureTest methodsFor: 'time measures' stamp: 'PabloFromer 6/3/2012 20:43'!
sixtySeconds
	
	
	|unit|
	
	unit:= self second.
	
	^(Measure unit: unit amount: 60). ! !

!MeasureTest methodsFor: 'temperature measures' stamp: 'HernanWilkinson 4/25/2011 17:28'!
temperatureExamples

	^self objectsInCategoryOf: thisContext method selector.
	
! !

!MeasureTest methodsFor: 'temperature units' stamp: 'HernanWilkinson 4/25/2011 17:28'!
temperatureUnits

	^self objectsInCategoryOf: thisContext method selector.
	
! !

!MeasureTest methodsFor: '1 - Test equality' stamp: 'HernanWilkinson 4/25/2011 16:05'!
test01ObjectsRepresentingSameMeasureShouldBeEqual

	self assert: self oneMeter = self oneMeter.
	self assert: self oneSecond = self oneSecond.
	self assert: self oneCelsius = self oneCelsius.
	

! !

!MeasureTest methodsFor: '1 - Test equality' stamp: 'HernanWilkinson 4/25/2011 16:06'!
test02DifferentMeasuresShouldNotBeEqual

	self deny: self oneMeter = self twoMeters.
	self deny: self oneSecond = self sixtySeconds.
	self deny: self oneCelsius = self zeroCelsius.
! !

!MeasureTest methodsFor: '1 - Test equality' stamp: 'HernanWilkinson 4/25/2011 16:07'!
test03SameMeasureExpressWithDifferentUnitsShouldBeEqual

	self assert: self oneMeter = self oneHundredCentimeters.
	self assert: self twoMeters = self twoHundredCentimeters.
	self assert: self oneMinute = self sixtySeconds.
	self assert: self zeroCelsius = self thirtyTwoFahrenheits.
! !

!MeasureTest methodsFor: '1 - Test equality' stamp: 'HernanWilkinson 4/25/2011 16:08'!
test03aMeasuresOfDifferentDomainShouldNotBeEqual

	self deny: self oneMeter = self oneSecond.
	self deny: self oneMeter = self oneCelsius.
	self deny: self oneCelsius = self oneSecond
! !

!MeasureTest methodsFor: '1 - Test equality' stamp: 'HernanWilkinson 4/25/2011 16:09'!
test04EqualsShouldBeCommutative

	self assert: (self oneMeter = self oneHundredCentimeters and: [ self oneHundredCentimeters = self oneMeter ]).
	self assert: (self twoMeters = self twoHundredCentimeters and:  [ self twoHundredCentimeters = self twoMeters ]).
	self assert: (self oneMinute = self sixtySeconds and: [self sixtySeconds = self oneMinute]).
	self assert: (self zeroCelsius = self thirtyTwoFahrenheits and: [ self thirtyTwoFahrenheits = self zeroCelsius]).
! !

!MeasureTest methodsFor: '2 - Converting/accessing/printing' stamp: 'PabloFromer 6/3/2012 21:41'!
test05UnitStringRepresentationShouldBeHumanReadable

	self assert: self meter printString = 'meter'.
	self assert: self centimeter printString = 'centimeter'.
	self assert: self decimeter printString = 'decimeter'.
	self assert: self millimeter printString = 'millimeter'.
	self assert: self decameter printString = 'decameter'.
	self assert: self hectometer printString = 'hectometer'.
	self assert: self kilometer printString = 'kilometer'.	
	
	self assert: self celsius printString = 'celsius'.	
	self assert: self fahrenheit printString = 'fahrenheit'.	

	self assert: self day printString = 'day'.	
	self assert: self hour printString = 'hour'.	
	self assert: self minute printString = 'minute'.	
	self assert: self second printString = 'second'.	

! !

!MeasureTest methodsFor: '2 - Converting/accessing/printing' stamp: 'PabloFromer 6/3/2012 22:01'!
test06MeasureStringRepresentationShouldBeHumanReadable

	self assert: (self oneMeter printString = '1 meter').
	self assert: self twoMeters printString = '2 meters'.
	self assert: self oneHundredCentimeters printString = '100 centimeters'.
	
	self assert: self oneMinute printString = '1 minute'.
	self assert: self sixtySeconds printString = '60 seconds'.
	
	self assert: self zeroCelsius printString = '0 celsius'.
	self assert: self thirtyTwoFahrenheits printString = '32 fahrenheits'.
	
	
! !

!MeasureTest methodsFor: '2 - Converting/accessing/printing' stamp: 'HernanWilkinson 4/25/2011 16:14'!
test07aMeasureAmountShouldBeTheExpectedAmount

	self assert: self oneMeter amount = 1.
	self assert: self twoMeters amount = 2.
	self assert: self oneHundredCentimeters amount = 100.
	self assert: self twoHundredCentimeters amount = 200.

	self assert: self oneMinute amount = 1.
	self assert: self sixtySeconds amount = 60.
	
	self assert: self zeroCelsius amount = 0.
	self assert: self thirtyTwoFahrenheits amount = 32.
! !

!MeasureTest methodsFor: '2 - Converting/accessing/printing' stamp: 'HernanWilkinson 4/25/2011 16:15'!
test07bMeasureUnitShouldBeTheExpectedUnit

	self assert: self oneMeter unit = self meter.
	self assert: self twoMeters unit = self meter.
	self assert: self oneHundredCentimeters unit = self centimeter.
	self assert: self twoHundredCentimeters unit = self centimeter.
	
	self assert: self oneMinute unit = self minute.
	self assert: self sixtySeconds unit = self second.
	
	self assert: self zeroCelsius unit = self celsius.
	self assert: self thirtyTwoFahrenheits unit = self fahrenheit
! !

!MeasureTest methodsFor: '2 - Converting/accessing/printing' stamp: 'HernanWilkinson 4/25/2011 16:15'!
test07cDifferentMeasureUnitsShouldBeDifferent

	| unitsGroupedByItself |
	
	unitsGroupedByItself := self units groupedBy: [ :aUnit | aUnit ].
	self assert: (unitsGroupedByItself allSatisfy: [:aCollectionOfUnit | aCollectionOfUnit size = 1] )
! !

!MeasureTest methodsFor: '2 - Converting/accessing/printing' stamp: 'HernanWilkinson 4/25/2011 16:03'!
test07dDistancesCanBeConvertedToDifferentUnits

	"A partir de los test de igualdad, se deben haber dado cuenta que debemos poder
	convertir distancias a distintas unidades... y que ademas dos distancias de distintas unidades
	pueden ser iguales... Por lo tanto por qu no hacer esta caracteristica de las distancias explicita?
	Veran que luego se torna muy util
	
	Respecto de como esta escrito el test, para no tener que escribir los assert para oneMeter y oneHundredCentimeters 
	creo un array y lo recorro... Se podria escribir menos aun en este test?
	
	Y por qu se asserta sobre el amount y luego la unidad y no directametne sobre la distancia completa. ej:
	self assert: (aDistance converTo: self meter) = self oneMeter
	"
	
	(Array with: self oneMeter with: self oneHundredCentimeters) do: [ :aDistance |
		self assert: (aDistance convertTo: self meter) amount = 1.
		self assert: (aDistance convertTo: self meter) unit = self meter.
		
		self assert: (aDistance convertTo: self decimeter) amount = 10.
		self assert: (aDistance convertTo: self decimeter) unit = self decimeter.
		
		self assert: (aDistance convertTo: self centimeter) amount = 100.
		self assert: (aDistance convertTo: self centimeter) unit = self centimeter.
		
		self assert: (aDistance convertTo: self millimeter) amount = 1000.
		self assert: (aDistance convertTo: self millimeter) unit = self millimeter.
	
		self assert: (aDistance convertTo: self decameter) amount = (1/10).
		self assert: (aDistance convertTo: self decameter) unit = self decameter.
		
		self assert: (aDistance convertTo: self hectometer) amount = (1/100).
		self assert: (aDistance convertTo: self hectometer) unit = self hectometer.
	
		self assert: (aDistance convertTo: self kilometer) amount = (1/1000).
		self assert: (aDistance convertTo: self kilometer) unit = self kilometer ]! !

!MeasureTest methodsFor: '2 - Converting/accessing/printing' stamp: 'HernanWilkinson 4/25/2011 16:17'!
test07eTimeMeasuresCanBeConvertedToDifferentUnits

	self assert: (self oneMinute convertTo: self second) = self sixtySeconds.
	self assert: (self sixtySeconds convertTo: self minute) = self oneMinute.

	self assert: (self oneHour convertTo: self minute) = self sixtyMinutes.
	self assert: (self sixtyMinutes convertTo: self hour) = self oneHour.

	self assert: (self oneDay convertTo: self hour) = self twentyFourHours.
	self assert: (self twentyFourHours convertTo: self day) = self oneDay.

! !

!MeasureTest methodsFor: '2 - Converting/accessing/printing' stamp: 'HernanWilkinson 4/25/2011 16:17'!
test07fTemperatureMeasuresCanBeConvertedToDifferentUnits

	self assert: (self zeroCelsius convertTo: self fahrenheit) = self thirtyTwoFahrenheits.
	self assert: (self thirtyTwoFahrenheits convertTo: self celsius) = self zeroCelsius.

! !

!MeasureTest methodsFor: '2 - Converting/accessing/printing' stamp: 'PabloFromer 6/4/2012 00:40'!
test07gMeasuresCanNotBeConvertedToUnitsOfDifferentDomain

"self should: [ self oneMeter convertTo: self second ] raise: Error withExceptionDo: [ :anError | self assert: anError messageText = self canNotConvertMeasurementException]."
	self should: [ self oneMeter convertTo: self second ]  raise: self canNotConvertMeasurementException.
	self should: [ self oneSecond convertTo: self celsius ] raise: self canNotConvertMeasurementException.
	self should: [ self oneCelsius convertTo: self centimeter ] raise: self canNotConvertMeasurementException.


! !

!MeasureTest methodsFor: '3 - Test equality - implementation details' stamp: 'HernanWilkinson 4/25/2011 16:20'!
test08aComparingMeasuresWithOtherObjectsShouldReturnFalse

	"El igual no debe generar ningun tipo de error por mas que se comparen objetos
	de distintos tipos
	Fijense como se crea una instancia de Object sin ningun tipo de problema... qu representa?"

	self deny: self oneMeter = Object new.
	self deny: Object new = self oneMeter! !

!MeasureTest methodsFor: '3 - Test equality - implementation details' stamp: 'HernanWilkinson 4/25/2011 16:21'!
test08bMeasuresThatAreEqualShouldReturnTheSameHash

	"Debido a como funcionan los diccionarios, cuando dos objetos son iguales deben
	tener es mismo hash. La inversa no es verdad"
	
	self assert: self oneMeter hash=self oneMeter hash.
	self assert: self oneMeter hash=self oneHundredCentimeters hash.
	self assert: self oneMinute hash = self sixtySeconds hash.
	self assert: self zeroCelsius hash = self thirtyTwoFahrenheits hash.
	
	
! !

!MeasureTest methodsFor: '3 - Test equality - implementation details' stamp: 'HernanWilkinson 4/25/2011 16:21'!
test08cCopiesOfTheSameMeasureShouldBeEqual

	"Dos objetos que ocupan distintas posiciones de memoria pero representan
	la misma entidad de la realidad, deben devolver true cuando se los compara por igual, puesto
	que representan la misma entidad de la realidad.
	Por lo tanto, el mensaje #= significa 'representan el mismo ente de la realidad?'
	mientras que el mensaje #== significa ocupan la misma posicion de memoria?"

	| originalOneMeter copiedOneMeter |

	"Al objeto oneMeter lo copio para asegurarme de tener uno oneMeter en otra posicion de memoria"

	originalOneMeter := self oneMeter.
	copiedOneMeter := originalOneMeter copy.

	"Me aseguro que ocupen distinta posicion de memoria"
	self deny: originalOneMeter == copiedOneMeter.

	"Ahora me aseguro que sean iguales"
	self assert: originalOneMeter = copiedOneMeter! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:31'!
test09AddingMeasuresOfSameUnitShouldReturnAMeasureRepresentingTheAddingMeasures

	self assert: (self oneMeter + self oneMeter) = self twoMeters.
	self assert: (self oneSecond + self oneSecond) = self twoSeconds.
	self assert: (self oneCelsius + self oneCelsius) = self twoCelsius.
	

! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:32'!
test10AddingMeasuresOfDifferentUnitShouldReturnAMeasureRepresentingTheAddingMeasures

	self assert: (self oneMeter + self oneHundredCentimeters) = self twoMeters.
	self assert: (self oneMeter + self oneHundredCentimeters) = self twoHundredCentimeters.
	self assert: (self oneSecond + self oneMinute) = self sixtyOneSeconds.

! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:48'!
test10aCanNotAddMeasuresOfDifferentDomain

	self should: [ self oneMeter + self oneSecond ] raise: self canNotConvertMeasurementException 

! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:32'!
test11AddingMeasuresIsCommutative

	self assert: (self oneMeter + self twoMeters)=(self twoMeters + self oneMeter).
	self assert: (self oneSecond + self twoSeconds)=(self twoSeconds + self oneSecond).
	self assert: (self oneCelsius + self twoCelsius)=(self twoCelsius + self oneCelsius)
	


! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:33'!
test12AddingMeasuresIsAssociative

	self assert: 
		((self oneMeter + self oneMeter) + self twoMeters) = 
		(self oneMeter + (self oneMeter + self twoMeters)).

	self assert: 
		((self oneSecond + self oneSecond) + self twoSeconds) = 
		(self oneSecond + (self oneSecond + self twoSeconds)).

	self assert: 
		((self oneCelsius + self oneCelsius) + self twoCelsius) = 
		(self oneCelsius + (self oneCelsius + self twoCelsius)).

		

! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:35'!
test12aAddingMeasuresKeepsMeasureImmutability

	| oneMeter oneSecond oneCelsius |
	
	oneMeter := self oneMeter.
	oneMeter + oneMeter.
	
	self assert: oneMeter = self oneMeter.
	
	oneSecond := self oneSecond.
	oneSecond + oneSecond.
	
	self assert: oneSecond = self oneSecond.
	
	oneCelsius := self oneCelsius.
	oneCelsius + oneCelsius.
	
	self assert: oneCelsius = self oneCelsius 



! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'Maffongelli 6/4/2012 11:13'!
test13aNegatingAMeasureNegatesItsAmount

	(Array with: self oneMeter with: self twoSeconds with: self thirtyTwoFahrenheits) do: [:aMeasure |
		self assert: aMeasure negated amount=aMeasure amount negated ]
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:36'!
test13bNegatingAMeasureIsReflective

	(Array with: self oneMeter with: self twoSeconds with: self thirtyTwoFahrenheits) do: [:aMeasure |
		self assert: aMeasure negated negated=aMeasure ].
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:37'!
test14SubtractingMeasuresOfSameUnitShouldReturnAMeasureRepresentingTheSubtractedMeasures

	self assert: (self twoMeters - self oneMeter) = self oneMeter.
	self assert: (self oneMeter - self oneMeter) = self zeroMeters.
	self assert: (self oneMeter - self twoMeters) = self oneMeter negated.
	
	self assert: (self twoSeconds - self oneSecond) = self oneSecond.
	self assert: (self oneSecond - self oneSecond) = self zeroSeconds.
	self assert: (self oneSecond - self twoSeconds) = self oneSecond negated.
	
	self assert: (self twoCelsius - self oneCelsius) = self oneCelsius.
	self assert: (self oneCelsius - self oneCelsius) = self zeroCelsius.
	self assert: (self oneCelsius - self twoCelsius) = self oneCelsius negated.
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:38'!
test15SubtractingMeasuresOfDifferentUnitShouldReturnAMeasureRepresentingTheSubtractedMeasures

	self assert: (self twoMeters - self oneHundredCentimeters) = self oneMeter.
	self assert: (self oneMeter - self oneHundredCentimeters) = self zeroMeters.
	self assert: (self oneHundredCentimeters - self twoMeters) = self oneMeter negated.
	
	self assert: (self sixtyOneSeconds - self oneMinute) = self oneSecond.
	

	! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:48'!
test15aCanNotSubtractMeasuresOfDifferentDomain

	self should: [ self oneMeter - self oneSecond ] raise: self canNotConvertMeasurementException 

! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:38'!
test16SubtractingMeasuresIsAssociative

	self assert: 
		((self twoMeters - self oneMeter) - self oneMeter) = 
		(self twoMeters - (self oneMeter + self oneMeter)).

	self assert: 
		((self twoSeconds - self oneSecond) - self oneSecond) = 
		(self twoSeconds - (self oneSecond + self oneSecond)).	

	

	! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:39'!
test16aSubtractingMeasuresKeepsMeasuresImmutability

	| oneMeter oneSecond oneCelsius |
	
	oneMeter := self oneMeter.
	oneMeter - oneMeter.
	
	self assert: oneMeter = self oneMeter.
	
	oneSecond := self oneSecond.
	oneSecond - oneSecond.
	
	self assert: oneSecond = self oneSecond.
	
	oneCelsius := self oneCelsius.
	oneCelsius - oneCelsius.
	
	self assert: oneCelsius = self oneCelsius 


! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:40'!
test17aMeasuresCanBeMultiplyByAnInteger

	self assert: self oneMeter * 1 = self oneMeter.
	self assert: self oneMeter * 2 = self twoMeters.
	self assert: self oneMeter * 0 = self zeroMeters.

	self assert: self oneSecond * 1 = self oneSecond.
	self assert: self oneSecond * 2 = self twoSeconds.
	self assert: self oneSecond * 0 = self zeroSeconds.

	self assert: self oneCelsius * 1 = self oneCelsius.
	self assert: self oneCelsius * 2 = self twoCelsius.
	self assert: self oneCelsius * 0 = self zeroCelsius.
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:40'!
test17bMeasuresCanBeMultiplyByAFraction

	self assert: (self oneMeter * (2/3)) amount = (2/3).
	self assert: (self twoMeters * (2/3)) amount = (2 * (2/3)).

	self assert: (self oneSecond * (2/3)) amount = (2/3).
	self assert: (self twoSeconds * (2/3)) amount = (2 * (2/3)).

	self assert: (self oneCelsius * (2/3)) amount = (2/3).
	self assert: (self twoCelsius * (2/3)) amount = (2 * (2/3)).
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:41'!
test17cMeasuresCanBeMultiplyByAFloat

	self assert: (self oneMeter * 2.5) amount = 2.5.
	self assert: (self twoMeters * 2.5) amount = (2 * 2.5).

	self assert: (self oneSecond * 2.5) amount = 2.5.
	self assert: (self twoSeconds * 2.5) amount = (2 * 2.5).

	self assert: (self oneCelsius * 2.5) amount = 2.5.
	self assert: (self twoCelsius * 2.5) amount = (2 * 2.5).
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'PabloFromer 6/6/2012 02:28'!
test17dMeasuresCanBeMultiplyByAMeasureOfSameDomain

	self assert: (self oneMeter * self oneMeter) = self oneSquaredMeter.
	self assert: (self twoMeters * self twoMeters) = self fourSquaredMeters.
	self assert: (self twoMeters * self twoMeters * self twoMeters) = self eightCubicMeters.

	self assert: (self oneSecond * self oneSecond) = self oneSquaredSecond.
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:53'!
test17eMeasuresCanBeMultiplyByAMeasureOfDifferentDomain

	self assert: (self oneMeter * self twoSeconds) = self twoMetersTimesSecond.
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:42'!
test18MultiplicationKeepsMeasureImmutability

	| oneMeter oneSecond oneCelsius |
	
	oneMeter := self oneMeter.
	oneMeter * 2.
	
	self assert: oneMeter = self oneMeter.
	
	oneSecond := self oneSecond.
	oneSecond * 2.
	
	self assert: oneSecond = self oneSecond.
	
	oneCelsius := self oneCelsius.
	oneCelsius * 2.
	
	self assert: oneCelsius = self oneCelsius 




! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:49'!
test19aMeasuresCanBeDividedByAnInteger

	self assert: (self oneMeter / 1) = self oneMeter.
	self assert: (self twoMeters / 2) = self oneMeter.

	self assert: (self oneSecond / 1) = self oneSecond.
	self assert: (self twoSeconds / 2) = self oneSecond.

	self assert: (self oneCelsius / 1) = self oneCelsius.
	self assert: (self twoCelsius / 2) = self oneCelsius.

! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:50'!
test19bMeasuresCanBeDividedByAFraction

	self assert: (self twoMeters / (1/3)) amount = 6.
	self assert: (self twoSeconds / (1/3)) amount = 6.
	self assert: (self twoCelsius / (1/3)) amount = 6.




! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:50'!
test19cMeasuresCanBeDividedByAFloat

	self assert: (self twoMeters / 2.5) amount = (2/2.5).
	self assert: (self twoSeconds / 2.5) amount = (2/2.5).
	self assert: (self twoCelsius / 2.5) amount = (2/2.5).




! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:52'!
test19dMeasuresCanBeDividedByAMeasureOfSameDomain

	self assert: (self oneMeter / self oneMeter) = 1.
	self assert: (self twoMeters / self oneMeter) = 2.
		
	self assert: (self oneSecond / self oneSecond) = 1.
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'PabloFromer 6/6/2012 02:39'!
test19eMeasuresCanBeDividedByAMeasureOfDifferentDomain

	self assert: (self twoMeters / self oneSecond) = self twoMetersBySecond.
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:54'!
test20DivisionKeepsMeasureImmutability

	| oneMeter oneSecond oneCelsius |
	
	oneMeter := self oneMeter.
	oneMeter / 2.
	
	self assert: oneMeter = self oneMeter.
	
	oneSecond := self oneSecond.
	oneSecond / 2.
	
	self assert: oneSecond = self oneSecond.
	
	oneCelsius := self oneCelsius.
	oneCelsius / 2.
	
	self assert: oneCelsius = self oneCelsius 


! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'efo 5/2/2011 20:56'!
test20aFinalSpeedCanBeRepresentedWithMeasures

	self assert: 
		(self finalSpeedStartingAt: self oneMeterPerSecond accelerating: self twoMetersPerSquaredSecond during: self sixtySeconds) = 
		(self oneMeterPerSecond + (self twoMetersPerSquaredSecond * self sixtySeconds)).
		
	self assert: (self finalSpeedStartingAt: self oneMeterPerSecond accelerating: self twoMetersPerSquaredSecond during: self sixtySeconds) = self oneHundredTwentyOneMetersPerSecond 
	
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 17:00'!
test20bMultiplyingANumberByAUnitReturnsAMeasure

	self assert: (1*self meter) = self oneMeter.
	self assert: (2*self second) = self twoSeconds.
! !

!MeasureTest methodsFor: '5 - Test magnitude protocol' stamp: 'HernanWilkinson 4/25/2011 17:02'!
test21MeasuresProvideATotalOrder

	self assert: (self measureExamples allSatisfy: [ :aMeasure | aMeasure < (aMeasure + (1* aMeasure unit))])

! !

!MeasureTest methodsFor: '5 - Test magnitude protocol' stamp: 'HernanWilkinson 4/25/2011 17:03'!
test22MeasuresCanBeComparedWithGreaterThan

	self assert: (self measureExamples allSatisfy: [ :aMeasure | (aMeasure + (1 * aMeasure unit)) > aMeasure ])
! !

!MeasureTest methodsFor: '5 - Test magnitude protocol' stamp: 'HernanWilkinson 4/25/2011 17:04'!
test23MeasuresCanBeComparedWithLessThanOrEqual

	self assert: (self measureExamples allSatisfy: [ :aMeasure | aMeasure <= aMeasure and: [aMeasure <= (aMeasure + (1 * aMeasure unit))]])

	
! !

!MeasureTest methodsFor: '5 - Test magnitude protocol' stamp: 'HernanWilkinson 4/25/2011 17:05'!
test24MeasuresCanBeComparedWithGreaterThanOrEqual

	self assert: (self measureExamples allSatisfy: [ :aMeasure | aMeasure >= aMeasure and: [(aMeasure + (1 * aMeasure unit)) >= aMeasure]])! !

!MeasureTest methodsFor: '5 - Test magnitude protocol' stamp: 'HernanWilkinson 4/25/2011 17:05'!
test25MaxReturnsTheMaximumBetweenTwoMeasures

	self assert: (self twoMeters max: self oneMeter) = self twoMeters.
	self assert: (self oneMeter max: self twoMeters) = self twoMeters.

	self assert: (self twoMeters max: self oneHundredCentimeters) = self twoMeters.
	self assert: (self oneHundredCentimeters max: self twoMeters) = self twoMeters.

	
! !

!MeasureTest methodsFor: '5 - Test magnitude protocol' stamp: 'HernanWilkinson 4/25/2011 17:05'!
test26MinReturnsTheMinimumBetweenTwoMeasures

	self assert: (self twoMeters min: self oneMeter) = self oneMeter.
	self assert: (self oneMeter min: self twoMeters) = self oneMeter.

	self assert: (self twoMeters min: self oneHundredCentimeters) = self oneHundredCentimeters.
	self assert: (self oneHundredCentimeters min: self twoMeters) = self oneHundredCentimeters.

	
! !

!MeasureTest methodsFor: '5 - Test magnitude protocol' stamp: 'HernanWilkinson 4/25/2011 17:05'!
test27BetweenAndReturnsTrueIfAMeasureIsBetweenOtherTwoMeasures

	self assert: (self oneMeter between: self zeroMeters and: self twoMeters).
	self assert: (self oneHundredCentimeters between: self zeroMeters and: self twoMeters).
	
	self deny: (self twoMeters between: self zeroMeters and: self oneMeter).
	self deny: (self twoHundredCentimeters between: self zeroMeters and: self oneMeter).

! !

!MeasureTest methodsFor: '6 - Test interval protocol' stamp: 'HernanWilkinson 4/25/2011 17:09'!
test28IntervalOfMeasuresCanBeCreatedBecauseTheyDefineATotalOrder

	self assert: (self oneMeter to: self twoMeters) first=self oneMeter.
	self assert: (self oneMeter to: self twoMeters) last=self twoMeters.
	self assert: (self oneMeter to: self twoMeters) increment =self oneMeter.	

	self assert: (self oneHundredCentimeters to: self twoMeters) first=self oneMeter.
	self assert: (self oneHundredCentimeters to: self twoMeters) last=self twoMeters.
	self assert: (self oneHundredCentimeters to: self twoMeters) increment=self oneCentimeter.	
	! !

!MeasureTest methodsFor: '6 - Test interval protocol' stamp: 'HernanWilkinson 4/25/2011 17:16'!
test29MeasureIntervalStepCanBeAnyMeasure

	self assert: (self zeroMeters to: (self twoMeters * 2) by: self twoMeters) first=self zeroMeters.
	self assert: (self zeroMeters to: (self twoMeters * 2) by: self twoMeters) last=(self twoMeters * 2).
	self assert: (self zeroMeters to: (self twoMeters * 2) by: self twoMeters) increment=self twoMeters.

	self assert: ((self twoMeters * 2) to: self zeroMeters by: self twoMeters negated) first=(self twoMeters * 2).
	self assert: ((self twoMeters * 2) to: self zeroMeters by: self twoMeters negated) last=self zeroMeters.
	self assert: ((self twoMeters * 2) to: self zeroMeters by: self twoMeters negated) increment=self twoMeters negated

	! !

!MeasureTest methodsFor: '6 - Test interval protocol' stamp: 'HernanWilkinson 4/25/2011 17:17'!
test30MeasureIntervalSizeReturnsTheNumberOfMeasuresOfTheInterval

	self assert: (self zeroMeters to: (self twoMeters * 2) ) size =5.
	self assert: ((self twoMeters * 2) to: self zeroMeters by: self oneMeter negated) size =5.
! !

!MeasureTest methodsFor: '6 - Test interval protocol' stamp: 'HernanWilkinson 4/25/2011 17:17'!
test31MeasureIntervalCanBeEnumeratedUsingDo

	| value |
	
	value := 1.
	(self oneMeter to: (self oneMeter * 10)) do: [ :aMeasure |
		self assert: aMeasure amount = value.
		value := value + 1 ].
	
	self assert: value = 11
! !

!MeasureTest methodsFor: '6 - Test interval protocol' stamp: 'HernanWilkinson 4/25/2011 17:18'!
test32MeasureIntervalCanBeEnumeratedWithSpecificStep

	| value initialDistance byDistance finalDistance |
	
	initialDistance := self oneMeter.
	byDistance := self twoMeters.
	finalDistance := initialDistance * 9.
	value := initialDistance amount.
		
	(initialDistance to: finalDistance by:  byDistance) do: [ :aMeasure |
		self assert: aMeasure amount = value.
		value := value + byDistance amount ].
	
	self assert:  value = (finalDistance amount + byDistance amount)
! !

!MeasureTest methodsFor: '7 - Polymorphic zero' stamp: 'HernanWilkinson 4/25/2011 17:20'!
test33NothingOnAnyDomainAreEquals

	self assert: self zeroMeters = self zeroSeconds.
	self assert: self zeroSeconds = self zeroKelvin.
	self assert: self zeroKelvin = self zeroMeters! !

!MeasureTest methodsFor: '7 - Polymorphic zero' stamp: 'PabloFromer 6/6/2012 03:36'!
test34ZeroCelsiusIsNotNothingOnTemperature

	self deny: self zeroCelsius = self zeroKelvin.
	self deny: self zeroKelvin = self zeroCelsius.
! !

!MeasureTest methodsFor: 'temperature measures' stamp: 'PabloFromer 6/3/2012 20:35'!
thirtyTwoFahrenheits
	
	|unit|
		
	unit:= self fahrenheit .	
	^(Measure unit: unit amount: 32). ! !

!MeasureTest methodsFor: 'time measures' stamp: 'HernanWilkinson 4/25/2011 17:29'!
timeExamples

	^self objectsInCategoryOf: thisContext method selector.
	
! !

!MeasureTest methodsFor: 'time units' stamp: 'HernanWilkinson 4/29/2011 17:10'!
timeUnits

	^self objectsInCategoryOf: thisContext method selector.
	
 ! !

!MeasureTest methodsFor: 'time measures' stamp: 'PabloFromer 6/3/2012 20:43'!
twentyFourHours
	
	
	|unit|
	
	unit:= self hour.
	
	^(Measure unit: unit amount: 24). ! !

!MeasureTest methodsFor: 'temperature measures' stamp: 'PabloFromer 6/3/2012 20:36'!
twoCelsius
		
	|unit|
	
	unit:= self celsius .
	
	^(Measure unit: unit amount: 2). ! !

!MeasureTest methodsFor: 'distance measures' stamp: 'PabloFromer 6/3/2012 20:34'!
twoHundredCentimeters
	
	|unit|
	
	unit:=self centimeter.
	
	
	^(Measure unit: unit amount: 200). ! !

!MeasureTest methodsFor: 'distance measures' stamp: 'PabloFromer 6/3/2012 20:34'!
twoMeters
	
	|unit|
	
	unit:= self meter.
	
	
	^(Measure unit: unit amount: 2). ! !

!MeasureTest methodsFor: 'compound measures' stamp: 'PabloFromer 6/6/2012 03:05'!
twoMetersBySecond

 	|unitsUp unitsDown|
	
	unitsUp:=(OrderedCollection new).
	
	unitsDown:=(OrderedCollection new).
	
	unitsUp add: self meter.
	
	unitsDown add: self second.
	 

	^CompundMeasure unitsUp: unitsUp unitsDown: unitsDown amount: 2. ! !

!MeasureTest methodsFor: 'compound measures' stamp: 'Maffongelli 6/6/2012 13:31'!
twoMetersPerSquaredSecond



 	|unitsUp unitsDown|
	
	unitsUp := (OrderedCollection new) .
	
	unitsUp add: self meter.
	
	unitsDown := (OrderedCollection new) .
	
	unitsDown add: self second.
	 
	unitsDown add: self second.

	^CompundMeasure unitsUp: unitsUp unitsDown: unitsDown amount: 2. ! !

!MeasureTest methodsFor: 'compound measures' stamp: 'PabloFromer 6/6/2012 02:37'!
twoMetersTimesSecond


 	|unitsUp unitsDown|
	
	unitsUp:=(OrderedCollection new).
	
	unitsUp add: self second.
	
	unitsUp add: self meter.
	 

	^CompundMeasure unitsUp: unitsUp unitsDown: (OrderedCollection new) amount: 2. ! !

!MeasureTest methodsFor: 'time measures' stamp: 'PabloFromer 6/3/2012 20:43'!
twoSeconds
	
	
	|unit|
	
	unit:= self second.
	
	^(Measure unit: unit amount: 2). ! !

!MeasureTest methodsFor: 'objects in category' stamp: 'HernanWilkinson 4/25/2011 17:34'!
units 

	^ OrderedCollection new
		addAll: self distanceUnits;
		addAll: self temperatureUnits;
		addAll: self timeUnits;
		yourself ! !

!MeasureTest methodsFor: 'temperature measures' stamp: 'PabloFromer 6/3/2012 20:36'!
zeroCelsius
		
	|unit|
	
	unit:= self celsius .
	
	^(Measure unit: unit amount: 0). ! !

!MeasureTest methodsFor: 'temperature measures' stamp: 'PabloFromer 6/3/2012 20:36'!
zeroKelvin
	
		
	|unit|
	
	unit:= self kelvin.
	
	^(Measure unit: unit amount: 0). ! !

!MeasureTest methodsFor: 'distance measures' stamp: 'PabloFromer 6/3/2012 20:34'!
zeroMeters

		
	|unit|
	
	unit:= self meter.
	
	^(Measure unit: unit amount: 0). ! !

!MeasureTest methodsFor: 'time measures' stamp: 'PabloFromer 6/3/2012 20:44'!
zeroSeconds
	
	
	|unit|
	
	unit:= self second.
	
	^(Measure unit: unit amount: 0). ! !

!SmallInteger methodsFor: '*ejerciciospoo-measure4' stamp: 'FromerMaffongelli 6/6/2012 16:20'!
* aNumber 
	"Primitive. Multiply the receiver by the argument and answer with the
	result if it is a SmallInteger. Fail if the argument or the result is not a
	SmallInteger. Essential. No Lookup. See Object documentation whatIsAPrimitive."

<primitive: 9>
    (aNumber isKindOf: MeasureUnit )
    ifTrue:[^Measure unit: aNumber amount: self].
   
(aNumber isKindOf: Measure )
    ifTrue:[^Measure unit: aNumber unit amount: self * aNumber amount ]
     ifFalse: [
    ^ super * aNumber
    ].! !

Object subclass: #MeasureUnit
	instanceVariableNames: 'name domain convertionFromSelfToNormalBlock convertionFromNormalToSelfBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-Measure4'!

!MeasureUnit class methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/6/2012 01:28'!
convertionFromSelfToNormalBlock: aBlock convertionFromNormalToSelfBlock: anotherBlock name: aName domain: aDomain

^(self new) convertionFromSelfToNormalBlock: aBlock convertionFromNormalToSelfBlock: anotherBlock name: aName domain: aDomain.! !

!MeasureUnit methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/6/2012 01:29'!
= anotherUnit

^(self name = anotherUnit name & self domain = anotherUnit domain & (self convertionFromNormalToSelfBlock value: 1) = (anotherUnit convertionFromNormalToSelfBlock value: 1) )! !

!MeasureUnit methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/3/2012 23:31'!
convertionFromNormalToSelfBlock

^convertionFromNormalToSelfBlock.! !

!MeasureUnit methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/3/2012 23:30'!
convertionFromSelfToNormalBlock

^convertionFromSelfToNormalBlock. ! !

!MeasureUnit methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/6/2012 01:28'!
convertionFromSelfToNormalBlock: aBlock convertionFromNormalToSelfBlock: anotherBlock name: aName domain: aDomain

convertionFromSelfToNormalBlock:= aBlock.
convertionFromNormalToSelfBlock:= anotherBlock.
name:= aName.
domain:= aDomain.! !

!MeasureUnit methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/3/2012 20:46'!
domain

^domain.! !

!MeasureUnit methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/3/2012 19:40'!
name

^name.! !

!MeasureUnit methodsFor: 'as yet unclassified' stamp: 'PabloFromer 6/3/2012 21:40'!
printString


	^self name.! !
